==> sc_retail_analysis-0.0.19-py3-none-any.whl: sc_retail_analysis/credit_card/credit_card_amount_analyzer.py <==
#  The MIT License (MIT)
#
#  Copyright (c) 2021. Scott Lau
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in all
#  copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#  SOFTWARE.

import logging

import numpy as np
import pandas as pd

from sc_retail_analysis.base_analyzer import BaseAnalyzer
from sc_retail_analysis.utils import ConfigUtils, ManifestUtils


class CreditCardAmountAnalyzer(BaseAnalyzer):
    """
    信用卡大额分期投放额分析
    """

    def __init__(self, *, excel_writer: pd.ExcelWriter):
        super().__init__(excel_writer=excel_writer)
        self._key_enabled = "retail.credit_card.credit_amount.enabled"
        self._key_business_type = "retail.credit_card.credit_amount.business_type"
        self._key_export_column_list = "retail.credit_card.credit_amount.sheet_config.export_column_list"

    def _read_config(self):
        config = ConfigUtils.get_config()
        # 信用卡大额分期报表文件路径
        self._src_filepath = config.get("retail.credit_card.credit_amount.source_file_path")
        # Sheet名称
        self._sheet_name = config.get("retail.credit_card.credit_amount.sheet_name")
        # 表头行索引
        self._header_row = config.get("retail.credit_card.credit_amount.sheet_config.header_row")
        # 员工工号列索引
        self._id_column = config.get("retail.credit_card.credit_amount.sheet_config.id_column")
        # 员工姓名列索引
        self._name_column = config.get("retail.credit_card.credit_amount.sheet_config.name_column")
        # 合计列索引
        self._total_balance_column = config.get("retail.credit_card.credit_amount.sheet_config.total_balance_column")
        # 生成的Excel中贷款余额的列名
        self._target_total_balance_column_name = config.get(
            "retail.credit_card.credit_amount.sheet_config.target_total_balance_column_name")

    def _read_src_file(self) -> pd.DataFrame:
        logging.getLogger(__name__).info("读取源文件：{}".format(self._src_filepath))
        data = pd.read_excel(self._src_filepath, sheet_name=self._sheet_name, header=self._header_row)
        self._id_column_name = data.columns[self._id_column]
        self._name_column_name = data.columns[self._name_column]
        self._total_balance_column_name = data.columns[self._total_balance_column]
        if not data.empty:
            # 工号前面的列可能包括合计，去除合计行
            for index in range(self._id_column):
                # 过滤合计行
                criterion = data[data.columns[index]].map(lambda x: x != '合计')
                data = data[criterion].copy()
        return data

    def _add_export_column_manifest_branch(self, origin_data: pd.DataFrame):
        if origin_data is None or origin_data.empty:
            return origin_data
        # 与花名册整合，添加花名册所在部门一列
        data = origin_data.merge(
            ManifestUtils.get_name_branch_data_frame(),
            how="left",
            left_on=[self._name_column_name],
            right_on=[ManifestUtils.get_name_column_name()]
        )
        return data

    def _rename_target_columns(self, *, data: pd.DataFrame) -> pd.DataFrame:
        df = data.rename(columns={
            self._total_balance_column_name: self._target_total_balance_column_name,
        })
        return df

    def _pivot_table(self, *, data: pd.DataFrame) -> pd.DataFrame:
        index_columns = [self._id_column_name, self._name_column_name]
        value_columns = [self._target_total_balance_column_name]
        if data.empty:
            return pd.DataFrame(columns=index_columns + value_columns)
        table = pd.pivot_table(data, values=value_columns,
                               index=index_columns,
                               aggfunc=np.sum, fill_value=0)
        return table

    def _after_pivot_table(self, *, data: pd.DataFrame) -> pd.DataFrame:
        return data.reset_index()

    def _merge_with_manifest(self, *, manifest_data: pd.DataFrame, data: pd.DataFrame) -> pd.DataFrame:
        logging.getLogger(__name__).info("与花名册合并...")
        merge_result = ManifestUtils.merge_with_manifest(manifest_data=manifest_data, data=data,
                                                         id_column_name=self._id_column_name,
                                                         name_column_name=self._name_column_name)
        return merge_result

    def _drop_duplicated_columns(self, *, data: pd.DataFrame) -> pd.DataFrame:
        return data.drop(columns=[self._id_column_name, self._name_column_name])

    def _add_target_columns(self) -> None:
        self._add_target_column(self._target_total_balance_column_name)

==> sc_retail_analysis-0.0.33-py3-none-any.whl: sc_retail_analysis/credit_card/credit_card_amount_analyzer.py <==
#  The MIT License (MIT)
#
#  Copyright (c) 2021. Scott Lau
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in all
#  copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#  SOFTWARE.

import logging

import numpy as np
import pandas as pd
from config42 import ConfigManager
from sc_analyzer_base import ManifestUtils

from sc_retail_analysis.analyzer.base_analyzer import BaseAnalyzer


class CreditCardAmountAnalyzer(BaseAnalyzer):
    """
    信用卡大额分期投放额分析
    """

    def __init__(self, *, config: ConfigManager, excel_writer: pd.ExcelWriter):
        super().__init__(config=config, excel_writer=excel_writer)
        self._key_enabled = "retail.credit_card.credit_amount.enabled"
        self._key_business_type = "retail.credit_card.credit_amount.business_type"
        self._key_export_column_list = "retail.credit_card.credit_amount.sheet_config.export_column_list"

    def _read_config(self, *, config: ConfigManager):
        # 信用卡大额分期报表文件路径
        self._src_filepath = config.get("retail.credit_card.credit_amount.source_file_path")
        # Sheet名称
        self._sheet_name = config.get("retail.credit_card.credit_amount.sheet_name")
        # 表头行索引
        self._header_row = config.get("retail.credit_card.credit_amount.sheet_config.header_row")
        # 员工工号列索引
        self._id_column = config.get("retail.credit_card.credit_amount.sheet_config.id_column")
        # 员工姓名列索引
        self._name_column = config.get("retail.credit_card.credit_amount.sheet_config.name_column")
        # 合计列索引
        self._total_balance_column = config.get("retail.credit_card.credit_amount.sheet_config.total_balance_column")
        # 生成的Excel中贷款余额的列名
        self._target_total_balance_column_name = config.get(
            "retail.credit_card.credit_amount.sheet_config.target_total_balance_column_name")

    def _read_src_file(self) -> pd.DataFrame:
        logging.getLogger(__name__).info("读取源文件：{}".format(self._src_filepath))
        data = pd.read_excel(self._src_filepath, sheet_name=self._sheet_name, header=self._header_row)
        self._id_column_name = data.columns[self._id_column]
        self._name_column_name = data.columns[self._name_column]
        self._total_balance_column_name = data.columns[self._total_balance_column]
        if not data.empty:
            # 工号前面的列可能包括合计，去除合计行
            for index in range(self._id_column):
                # 过滤合计行
                criterion = data[data.columns[index]].map(lambda x: x != '合计')
                data = data[criterion].copy()
        return data

    def _add_export_column_manifest_branch(self, origin_data: pd.DataFrame):
        if origin_data is None or origin_data.empty:
            return origin_data
        # 与花名册整合，添加花名册所在部门一列
        data = origin_data.merge(
            ManifestUtils.get_name_branch_data_frame(),
            how="left",
            left_on=[self._name_column_name],
            right_on=[ManifestUtils.get_name_column_name()]
        )
        return data

    def _rename_target_columns(self, *, data: pd.DataFrame) -> pd.DataFrame:
        df = data.rename(columns={
            self._total_balance_column_name: self._target_total_balance_column_name,
        })
        return df

    def _pivot_table(self, *, data: pd.DataFrame) -> pd.DataFrame:
        index_columns = [self._id_column_name, self._name_column_name]
        value_columns = [self._target_total_balance_column_name]
        if data.empty:
            return pd.DataFrame(columns=index_columns + value_columns)
        table = pd.pivot_table(data, values=value_columns,
                               index=index_columns,
                               aggfunc=np.sum, fill_value=0)
        return table

    def _after_pivot_table(self, *, data: pd.DataFrame) -> pd.DataFrame:
        return data.reset_index()

    def _merge_with_manifest(self, *, manifest_data: pd.DataFrame, data: pd.DataFrame) -> pd.DataFrame:
        logging.getLogger(__name__).info("与花名册合并...")
        merge_result = ManifestUtils.merge_with_manifest(manifest_data=manifest_data, data=data,
                                                         id_column_name=self._id_column_name,
                                                         name_column_name=self._name_column_name)
        return merge_result

    def _drop_duplicated_columns(self, *, data: pd.DataFrame) -> pd.DataFrame:
        return data.drop(columns=[self._id_column_name, self._name_column_name])

    def _add_target_columns(self) -> None:
        self._add_target_column(self._target_total_balance_column_name)
